"use strict";
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
Object.defineProperty(exports, "__esModule", { value: true });
var widgets_1 = require("@phosphor/widgets");
var apputils_1 = require("@jupyterlab/apputils");
var codeeditor_1 = require("@jupyterlab/codeeditor");
var codemirror_1 = require("@jupyterlab/codemirror");
var fileeditor_1 = require("@jupyterlab/fileeditor");
/**
 * The command IDs used by the codemirror plugin.
 */
var CommandIDs;
(function (CommandIDs) {
    CommandIDs.matchBrackets = 'codemirror:match-brackets';
    CommandIDs.changeKeyMap = 'codemirror:change-keyMap';
    CommandIDs.changeTheme = 'codemirror:change-theme';
})(CommandIDs || (CommandIDs = {}));
;
/**
 * The editor services.
 */
exports.servicesPlugin = {
    id: codeeditor_1.IEditorServices.name,
    provides: codeeditor_1.IEditorServices,
    activate: function () { return codemirror_1.editorServices; }
};
/**
 * The editor commands.
 */
exports.commandsPlugin = {
    id: 'jupyter.services.codemirror-commands',
    requires: [fileeditor_1.IEditorTracker, apputils_1.IMainMenu, apputils_1.ICommandPalette, apputils_1.IStateDB],
    activate: activateEditorCommands,
    autoStart: true
};
/**
 * Export the plugins as default.
 */
var plugins = [exports.commandsPlugin, exports.servicesPlugin];
exports.default = plugins;
/**
 * Set up the editor widget menu and commands.
 */
function activateEditorCommands(app, tracker, mainMenu, palette, state) {
    var commands = app.commands;
    var theme = codemirror_1.CodeMirrorEditor.DEFAULT_THEME;
    var keyMap = 'default';
    var matchBrackets = false;
    var id = 'codemirror:settings';
    // Fetch the initial state of the settings.
    state.fetch(id).then(function (settings) {
        if (!settings) {
            return;
        }
        if (typeof settings['theme'] === 'string') {
            commands.execute(CommandIDs.changeTheme, settings);
        }
        if (typeof settings['keyMap'] === 'string') {
            commands.execute(CommandIDs.changeKeyMap, settings);
        }
        if (typeof settings['matchBrackets'] === 'boolean') {
            if (settings['matchBrackets'] !== matchBrackets) {
                commands.execute(CommandIDs.matchBrackets);
            }
        }
    });
    /**
     * Save the codemirror settings state.
     */
    function saveState() {
        return state.save(id, { theme: theme, keyMap: keyMap, matchBrackets: matchBrackets });
    }
    /**
     * Handle the settings of new widgets.
     */
    tracker.widgetAdded.connect(function (sender, widget) {
        if (widget.editor instanceof codemirror_1.CodeMirrorEditor) {
            var cm = widget.editor.editor;
            cm.setOption('keyMap', keyMap);
            cm.setOption('theme', theme);
            cm.setOption('matchBrackets', matchBrackets);
        }
    });
    /**
     * A test for whether the tracker has an active widget.
     */
    function hasWidget() {
        return tracker.currentWidget !== null;
    }
    /**
     * Toggle editor matching brackets
     */
    function toggleMatchBrackets() {
        matchBrackets = !matchBrackets;
        tracker.forEach(function (widget) {
            var editor = widget.editor;
            if (editor instanceof codemirror_1.CodeMirrorEditor) {
                var cm = editor.editor;
                cm.setOption('matchBrackets', matchBrackets);
            }
        });
        return saveState();
    }
    /**
     * Create a menu for the editor.
     */
    function createMenu() {
        var menu = new widgets_1.Menu({ commands: commands });
        var themeMenu = new widgets_1.Menu({ commands: commands });
        var keyMapMenu = new widgets_1.Menu({ commands: commands });
        menu.title.label = 'Editor';
        themeMenu.title.label = 'Theme';
        keyMapMenu.title.label = 'Key Map';
        commands.addCommand(CommandIDs.changeTheme, {
            label: function (args) { return args['theme']; },
            execute: function (args) {
                theme = args['theme'] || codemirror_1.CodeMirrorEditor.DEFAULT_THEME;
                tracker.forEach(function (widget) {
                    if (widget.editor instanceof codemirror_1.CodeMirrorEditor) {
                        var cm = widget.editor.editor;
                        cm.setOption('theme', theme);
                    }
                });
                return saveState();
            },
            isEnabled: hasWidget,
            isToggled: function (args) { return args['theme'] === theme; }
        });
        commands.addCommand(CommandIDs.changeKeyMap, {
            label: function (args) {
                var title = args['keyMap'];
                return title === 'sublime' ? 'Sublime Text' : title;
            },
            execute: function (args) {
                keyMap = args['keyMap'] || 'default';
                tracker.forEach(function (widget) {
                    if (widget.editor instanceof codemirror_1.CodeMirrorEditor) {
                        var cm = widget.editor.editor;
                        cm.setOption('keyMap', keyMap);
                    }
                });
                return saveState();
            },
            isEnabled: hasWidget,
            isToggled: function (args) { return args['keyMap'] === keyMap; }
        });
        [
            'jupyter', 'default', 'abcdef', 'base16-dark', 'base16-light',
            'hopscotch', 'material', 'mbo', 'mdn-like', 'seti', 'the-matrix',
            'xq-light', 'zenburn'
        ].forEach(function (name) { return themeMenu.addItem({
            command: CommandIDs.changeTheme,
            args: { theme: name }
        }); });
        [
            'default', 'sublime', 'vim', 'emacs'
        ].forEach(function (name) { return keyMapMenu.addItem({
            command: CommandIDs.changeKeyMap,
            args: { keyMap: name }
        }); });
        menu.addItem({ command: 'editor:line-numbers' });
        menu.addItem({ command: 'editor:word-wrap' });
        menu.addItem({ command: CommandIDs.matchBrackets });
        menu.addItem({ type: 'separator' });
        menu.addItem({ type: 'submenu', submenu: keyMapMenu });
        menu.addItem({ type: 'submenu', submenu: themeMenu });
        return menu;
    }
    mainMenu.addMenu(createMenu(), { rank: 30 });
    commands.addCommand(CommandIDs.matchBrackets, {
        execute: toggleMatchBrackets,
        label: 'Match Brackets',
        isEnabled: hasWidget,
        isToggled: function () { return !matchBrackets; }
    });
    [
        'editor:line-numbers',
        'editor:line-wrap',
        CommandIDs.matchBrackets,
        'editor:create-console',
        'editor:run-code'
    ].forEach(function (command) { return palette.addItem({ command: command, category: 'Editor' }); });
}
